package org.choub.minecraft.launcher.task;

import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.net.URI;
import java.text.MessageFormat;

import org.apache.http.HttpEntity;
import org.apache.http.HttpResponse;
import org.apache.http.HttpStatus;
import org.apache.http.client.ClientProtocolException;
import org.apache.http.client.HttpClient;
import org.apache.http.client.methods.HttpGet;
import org.apache.http.impl.client.DefaultHttpClient;
import org.apache.log4j.Logger;
import org.choub.minecraft.launcher.net.IProgressListener;
import org.choub.minecraft.launcher.util.FileUtil;


public class DownloadTask extends BackgroundTask {

	private static Logger logger = Logger.getLogger(DownloadTask.class);

	private HttpClient client = new DefaultHttpClient();
	private URI uri;
	private String fileName;
	private long totalBytes = -1;
	private long transferredBytes = 0;

	private IProgressListener listener;

	public DownloadTask(URI uri, String fileName) {
		super();
		this.uri = uri;
		this.fileName = fileName;
	}

	public DownloadTask(URI uri, String fileName, IProgressListener listener) {
		this(uri, fileName);
		this.listener = listener;
	}

	public float getProgress() {
		return ((float) transferredBytes / totalBytes) * 100;
	}

	@Override
	public void executeTask() {
		logger.info(MessageFormat.format("Starting download of [{0}]", uri));
		HttpGet get = new HttpGet(uri);

		try {
			handleResponse(client.execute(get));
		} catch (ClientProtocolException e) {
			logger.error(MessageFormat.format("Unable to download [{0}], protocol not supported [{1}]", uri, uri.getScheme()), e);
		} catch (FileNotFoundException e) {
			logger.error(MessageFormat.format("Unable to create destination file [{0}]", fileName), e);
		} catch (IOException e) {
			logger.error(MessageFormat.format("Unable to download [{0}]", uri), e);
		}
	}

	private void handleResponse(HttpResponse response) throws IOException {
		if (response.getStatusLine().getStatusCode() != HttpStatus.SC_OK) {
			logger.warn(MessageFormat.format("Unable to download, incorrect status code [{1}]", response.getStatusLine().getStatusCode()));
		}

		HttpEntity entity = response.getEntity();
		if (entity != null) {
			totalBytes = entity.getContentLength();
			readFile(entity.getContent());
		}
	}

	private void readFile(InputStream in) throws IOException {

		OutputStream out = null;
		try {

			File workingDirectory = FileUtil.getWorkingDirectory(getContext().getConfiguration().getWorkingDirectoryName());
			File destination = new File(workingDirectory, fileName);
			out = new FileOutputStream(destination);

			int read = 0;
			byte[] bytes = new byte[1024];
			while ((read = in.read(bytes)) != -1) {
				this.transferredBytes += read;
				out.write(bytes, 0, read);
				logger.info(MessageFormat.format("Transfer progress: [{0}]", getProgress()));
			}
			logger.info(MessageFormat.format("File successfully downloaded to [{0}]", destination));
		} finally {
			if (in != null) {
				in.close();
			}
			if (out != null) {
				out.flush();
				out.close();
			}
		}
	}
}
